@@ -8,6 +8,7 @@ from nle.env import NLE
 from nle.nethack import actions as nethack_actions
 
 from nle_language_wrapper import NLELanguageWrapper
+from nle_language_wrapper.scripts import play
 
 
 def str_to_1d(string):
@@ -184,13 +185,12 @@ def fake_nethack_multiple_monsters_env(mocker):
 
 
 def test_message_spell_menu(fake_nle_env):
-
     menu = [
-        "    Choose which spell to cast                               ",
+        "   Choose which spell to cast                               ",
         "                                                             ",
-        "    Name                 Level Category     Fail Retention   ",
+        "   Name                 Level Category     Fail Retention   ",
         "   a - healing                1   healing        0%      100%",
-        "(end)                                                        ",
+        "   (end)                                                     ",
     ]
     empty_lines = [
         "                                                      ",
@@ -209,18 +209,20 @@ def test_message_spell_menu(fake_nle_env):
 
     expected_menu = (
         "Choose which spell to cast\n"
+        "\n"
         "Name                 Level Category     Fail Retention\n"
-        "a - healing                1   healing        0%      100%"
+        "a - healing                1   healing        0%      100%\n"
+        "(end)"
     )
     assert obsv["text_message"] == expected_menu
 
 
-def test_message_more(fake_nle_env):
+def test_message_more_end(fake_nle_env):
     things_that_are_here = [
         " Things that are here:      ",
-        "a goblin corpse             ",
-        "an iron skull cap           ",
-        "--More--                    ",
+        " a goblin corpse             ",
+        " an iron skull cap           ",
+        " --More--                    ",
     ]
     screen_map = [
         "         -----------------------------------------|           ",
@@ -242,6 +244,187 @@ def test_message_more(fake_nle_env):
     assert obsv["text_message"] == expected_menu
 
 
+def test_message_full_stop_end(fake_nle_env):
+    message = [" It's a wall. ", " "]
+
+    screen_map = [
+        "         -----------------------------------------|           ",
+        "         |               @                        |           ",
+    ]
+
+    # Fill with spaces for menu
+    tty_chars = strs_to_2d(message + screen_map, fill_value=32)
+
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+
+    expected_menu = "It's a wall."
+    assert obsv["text_message"] == expected_menu
+
+
+def test_message_bracket_end(fake_nle_env):
+    message = [" What do you want to drop? [$a-k or ?*] ", " "]
+
+    screen_map = [
+        "         -----------------------------------------|           ",
+        "         |               @                        |           ",
+    ]
+    # Fill with spaces for menu
+    tty_chars = strs_to_2d(message + screen_map, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_menu = "What do you want to drop? [$a-k or ?*]"
+    assert obsv["text_message"] == expected_menu
+
+
+def test_message_parenthesis_end(fake_nle_env):
+    message = [" Where do you want to travel to?  (For instructions type a '?') ", " "]
+
+    screen_map = [
+        "         -----------------------------------------|           ",
+        "         |               @                        |           ",
+    ]
+    # Fill with spaces for menu
+    tty_chars = strs_to_2d(message + screen_map, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_message = "Where do you want to travel to?  (For instructions type a '?')"
+    assert obsv["text_message"] == expected_message
+
+
+def test_message_multipage(fake_nle_env):
+    message = [
+        " Extended Commands List    ",
+        "                               ",
+        " a - Hide commands that don't autocomplete (those not marked with [A])   ",
+        " : - Search extended commands      ",
+        " ",
+        " Extended commands                     ",
+        " #                  perform an extended command  ",
+        " ?              [A] list all extended commands  ",
+        " adjust         [A] adjust inventory letters   ",
+        " annotate       [A] name current level  ",
+        " (1 of 5) ",
+    ]
+
+    tty_chars = strs_to_2d(message, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_message = (
+        "Extended Commands List\n"
+        "\n"
+        "a - Hide commands that don't autocomplete (those not marked with [A])\n"
+        ": - Search extended commands\n"
+        "\n"
+        "Extended commands\n"
+        "#                  perform an extended command\n"
+        "?              [A] list all extended commands\n"
+        "adjust         [A] adjust inventory letters\n"
+        "annotate       [A] name current level\n"
+        "(1 of 5)"
+    )
+    assert obsv["text_message"] == expected_message
+
+
+def test_message_takeoffall(fake_nle_env):
+    message = [
+        " What type of things do you want to take off?    ",
+        " ",
+        "  a - All worn types                             ",
+        " b - Weapons ",
+        " c - Armor ",
+        " U - Items known to be Uncursed ",
+        " (end) ",
+    ]
+    screen_map = [
+        "         -----------------------------------------|           ",
+        "         |               @                        |           ",
+    ]
+    # Fill with spaces for menu
+    tty_chars = strs_to_2d(message + screen_map, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_message = (
+        "What type of things do you want to take off?\n"
+        "\n"
+        "a - All worn types\n"
+        "b - Weapons\n"
+        "c - Armor\n"
+        "U - Items known to be Uncursed\n"
+        "(end)"
+    )
+    assert obsv["text_message"] == expected_message
+
+
+def test_filter_map_from_conduct(fake_nle_env):
+    message = [
+        "                          Voluntary challenges:                  ",
+        "                           You have gone without food.              ",
+        "                           You have been an atheist.                ",
+        "              -----        You have never hit with a wielded weapon.",
+        "             #+....######  You have been a pacifist.                ",
+        "             #+....######  --More--                ",
+    ]
+    tty_chars = strs_to_2d(message, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_message = (
+        "Voluntary challenges:\n"
+        "You have gone without food.\n"
+        "You have been an atheist.\n"
+        "You have never hit with a wielded weapon.\n"
+        "You have been a pacifist.\n"
+        "--More--"
+    )
+    assert obsv["text_message"] == expected_message
+
+
+def test_filter_map_from_name(fake_nle_env):
+    message = [
+        "                              What do you want to name? ",
+        "                                                        ",
+        "---------                     m - a monster ",
+        "|........                     i - a particular object in inventory ",
+        "|.......|                     o - the type of an object in inventory  ",
+        "|....@.%|                     f - the type of an object upon the floor  ",
+        "|!...df.|                     d - the type of an object on discoveries list ",
+        "|.......+                     a - record an annotation for the current level ",
+        "--+------                     (end)",
+    ]
+    tty_chars = strs_to_2d(message, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_message = (
+        "What do you want to name?\n"
+        "\n"
+        "m - a monster\n"
+        "i - a particular object in inventory\n"
+        "o - the type of an object in inventory\n"
+        "f - the type of an object upon the floor\n"
+        "d - the type of an object on discoveries list\n"
+        "a - record an annotation for the current level\n"
+        "(end)"
+    )
+    assert obsv["text_message"] == expected_message
+
+
+def test_filter_map_travel(fake_nle_env):
+    message = ["doorway      ", "", "      ------------ ", "      |..........| "]
+    tty_chars = strs_to_2d(message, fill_value=32)
+    fake_nle_env.reset.return_value["tty_chars"] = tty_chars
+    dut = NLELanguageWrapper(fake_nle_env)
+    obsv = dut.reset()
+    expected_message = "doorway"
+    assert obsv["text_message"] == expected_message
+
+
 def test_create_env_real(real_nethack_env):
     dut = NLELanguageWrapper(real_nethack_env)
     dut.reset()
@@ -284,8 +467,20 @@ def test_step_real(real_nethack_env):
 def test_step_invalid_action(real_nethack_env):
     dut = NLELanguageWrapper(real_nethack_env)
     dut.reset()
-    with pytest.raises(ValueError):
-        dut.step("divide by zero")
+    with pytest.raises(
+        ValueError, match="is not recognized or not supported for this environment"
+    ):
+        dut.step("invalid action")
+
+
+def test_action_actions_maps_reflect_valid_actions(fake_nle_env):
+    fake_nle_env._actions = [nethack_actions.CompassDirection.N]
+    dut = NLELanguageWrapper(fake_nle_env)
+    assert dut.action_enum_index_map == {nethack_actions.CompassDirection.N: 0}
+    assert dut.action_str_enum_map == {
+        "north": nethack_actions.CompassDirection.N,
+        "k": nethack_actions.CompassDirection.N,
+    }
 
 
 def test_step_valid_action_not_supported(real_nethack_env):
@@ -308,7 +503,7 @@ def test_obsv_fake(fake_nle_env):
     expected_cursor = "Yourself a monk"
     expected_inventory = (
         "a: a blessed +1 quarterstaff (weapon in hands)\n"
-        "b: an uncursed +0 cloak of magic resistance (being worn)\n"
+        "b: an uncursed +0 cloak of magic resistance (being worn)"
     )
     expected_message = (
         "Hello Agent, welcome to NetHack!  You are a neutral gnomish Caveman."
@@ -322,27 +517,28 @@ def test_obsv_fake(fake_nle_env):
         "south, southwest, west, and northwest"
     )
     expected_text_blstats = (
-        "Strength:1/19\n"
-        "Dexterity:4\n"
-        "Constitution:5\n"
-        "Intelligence:6\n"
-        "Wisdom:7\n"
-        "Charisma:8\n"
-        "Depth:9\n"
-        "Gold:10\n"
-        "HP:11/12\n"
-        "Energy:1/13\n"
-        "AC:1\n"
-        "XP:7/10\n"
-        "Time:5\n"
-        "Position:66|14\n"
-        "Hunger:Fainting\nMonster Level:23\n"
-        "Encumbrance:Stressed\n"
-        "Dungeon Number:21\n"
-        "Level Number:10\n"
-        "Score:123\n"
-        "Alignment:Chaotic\n"
-        "Condition:Stoned Slimed Food Poisoning"
+        "Strength: 1/19\n"
+        "Dexterity: 4\n"
+        "Constitution: 5\n"
+        "Intelligence: 6\n"
+        "Wisdom: 7\n"
+        "Charisma: 8\n"
+        "Depth: 9\n"
+        "Gold: 10\n"
+        "HP: 11/12\n"
+        "Energy: 1/13\n"
+        "AC: 1\n"
+        "XP: 7/10\n"
+        "Time: 5\n"
+        "Position: 66|14\n"
+        "Hunger: Fainting\n"
+        "Monster Level: 23\n"
+        "Encumbrance: Stressed\n"
+        "Dungeon Number: 21\n"
+        "Level Number: 10\n"
+        "Score: 123\n"
+        "Alignment: Chaotic\n"
+        "Condition: Stoned Slimed Food Poisoning"
     )
 
     assert obsv["text_glyphs"] == expected_obsv
@@ -360,7 +556,7 @@ def test_blstats_condition_none(fake_nle_env):
     dut = NLELanguageWrapper(fake_nle_env)
     obsv = dut.reset()
 
-    expected_condition = "Condition:None"
+    expected_condition = "Condition: None"
 
     assert expected_condition in obsv["text_blstats"]
 
@@ -373,7 +569,7 @@ def test_blstats_condition_flying(fake_nle_env):
     dut = NLELanguageWrapper(fake_nle_env)
     obsv = dut.reset()
 
-    expected_condition = "Condition:Flying"
+    expected_condition = "Condition: Flying"
 
     assert expected_condition in obsv["text_blstats"]
 
@@ -383,7 +579,7 @@ def test_multiple_obsv_fake(fake_nethack_multiple_monsters_env):
     obsv = dut.reset()
     expected_inventory = (
         "a: a blessed +1 quarterstaff (weapon in hands)\n"
-        "b: an uncursed +0 cloak of magic resistance (being worn)\n"
+        "b: an uncursed +0 cloak of magic resistance (being worn)"
     )
     expected_message = (
         "Hello Agent, welcome to NetHack!  You are a neutral gnomish Caveman."
@@ -407,7 +603,7 @@ def test_step_fake(fake_nle_env):
     obsv = dut.reset()
     expected_inventory = (
         "a: a blessed +1 quarterstaff (weapon in hands)\n"
-        "b: an uncursed +0 cloak of magic resistance (being worn)\n"
+        "b: an uncursed +0 cloak of magic resistance (being worn)"
     )
     expected_message = (
         "Hello Agent, welcome to NetHack!  You are a neutral gnomish Caveman."
@@ -765,6 +961,11 @@ def test_wrapper_requires_all_keys(real_nethack_env):
         NLELanguageWrapper(real_nethack_env)
 
 
+def test_play(mocker):
+    with mocker.patch("builtins.input", return_value="north"):
+        play.main("NetHackChallenge-v0")
+
+
 def test_time_reset(real_nethack_env):
     dut = NLELanguageWrapper(real_nethack_env)
     baseline_runtime = min(timeit.repeat(real_nethack_env.reset, number=100, repeat=10))
