@@ -910,8 +910,11 @@ py::bytes NLELanguageObsv::text_inventory(py::array_t<uint8_t> inv_strs,
   for (uint64_t i = 0; i < x; i++) {
     std::string inv_letter(1, inv_letters_data[i]);
     std::string inv_str(reinterpret_cast<char *>(&(inv_strs_data[i * y])));
-    if (inv_letters_data[i] == 0) break;
-    output += inv_letter + ": " + inv_str + "\n";
+    if (inv_letters_data[i] != 0) {
+      if (i > 0) output += "\n";
+      output += inv_letter + ": " + inv_str;
+    } else
+      break;
   }
   return py::bytes(output);
 }
@@ -1081,31 +1084,31 @@ py::bytes NLELanguageObsv::text_blstats(py::array_t<int64_t> blstats) {
   }
 
   std::string output =
-      ("Strength:" + std::to_string(blstats_data[3]) + "/" +
+      ("Strength: " + std::to_string(blstats_data[3]) + "/" +
        std::to_string(blstats_data[2]) + "\n" +
-       "Dexterity:" + std::to_string(blstats_data[4]) + "\n" +
-       "Constitution:" + std::to_string(blstats_data[5]) + "\n" +
-       "Intelligence:" + std::to_string(blstats_data[6]) + "\n" +
-       "Wisdom:" + std::to_string(blstats_data[7]) + "\n" +
-       "Charisma:" + std::to_string(blstats_data[8]) + "\n" +
-       "Depth:" + std::to_string(blstats_data[12]) + "\n" +
-       "Gold:" + std::to_string(blstats_data[13]) + "\n" +
-       "HP:" + std::to_string(blstats_data[10]) + "/" +
+       "Dexterity: " + std::to_string(blstats_data[4]) + "\n" +
+       "Constitution: " + std::to_string(blstats_data[5]) + "\n" +
+       "Intelligence: " + std::to_string(blstats_data[6]) + "\n" +
+       "Wisdom: " + std::to_string(blstats_data[7]) + "\n" +
+       "Charisma: " + std::to_string(blstats_data[8]) + "\n" +
+       "Depth: " + std::to_string(blstats_data[12]) + "\n" +
+       "Gold: " + std::to_string(blstats_data[13]) + "\n" +
+       "HP: " + std::to_string(blstats_data[10]) + "/" +
        std::to_string(blstats_data[11]) + "\n" +
-       "Energy:" + std::to_string(blstats_data[14]) + "/" +
+       "Energy: " + std::to_string(blstats_data[14]) + "/" +
        std::to_string(blstats_data[15]) + "\n" +
-       "AC:" + std::to_string(blstats_data[16]) + "\n" +
-       "XP:" + std::to_string(blstats_data[18]) + "/" +
+       "AC: " + std::to_string(blstats_data[16]) + "\n" +
+       "XP: " + std::to_string(blstats_data[18]) + "/" +
        std::to_string(blstats_data[19]) + "\n" +
-       "Time:" + std::to_string(blstats_data[20]) + "\n" +
-       "Position:" + std::to_string(blstats_data[0]) + "|" +
-       std::to_string(blstats_data[1]) + "\n" + "Hunger:" + hunger_str + "\n" +
-       "Monster Level:" + std::to_string(blstats_data[17]) + "\n" +
-       "Encumbrance:" + encumberance_str + "\n" +
-       "Dungeon Number:" + std::to_string(blstats_data[23]) + "\n" +
-       "Level Number:" + std::to_string(blstats_data[24]) + "\n" +
-       "Score:" + std::to_string(blstats_data[9]) + "\n" +
-       "Alignment:" + alignment_str + "\n" + "Condition:" + condition);
+       "Time: " + std::to_string(blstats_data[20]) + "\n" +
+       "Position: " + std::to_string(blstats_data[0]) + "|" +
+       std::to_string(blstats_data[1]) + "\n" + "Hunger: " + hunger_str + "\n" +
+       "Monster Level: " + std::to_string(blstats_data[17]) + "\n" +
+       "Encumbrance: " + encumberance_str + "\n" +
+       "Dungeon Number: " + std::to_string(blstats_data[23]) + "\n" +
+       "Level Number: " + std::to_string(blstats_data[24]) + "\n" +
+       "Score: " + std::to_string(blstats_data[9]) + "\n" +
+       "Alignment: " + alignment_str + "\n" + "Condition: " + condition);
   return py::bytes(output);
 }
 
@@ -1126,31 +1129,31 @@ py::bytes NLELanguageObsv::text_message(py::array_t<uint8_t> tty_chars) {
   size_t rows = tty_chars_buffer.shape[0];
   size_t columns = tty_chars_buffer.shape[1];
   std::string output = "";
-  uint64_t blank_row_count = 0;
+  std::string first_row_str = "";
+  bool multipage_message = false;
+
+  std::string row_str(reinterpret_cast<char *>(&(tty_chars_data[0])), columns);
+  size_t indent = row_str.find_first_not_of(' ');
   for (uint64_t row_idx = 0; row_idx < rows; row_idx++) {
-    std::string row_str(
-        reinterpret_cast<char *>(&(tty_chars_data[row_idx * columns])),
-        columns);
+    std::string row_str(reinterpret_cast<char *>(
+                            &(tty_chars_data[indent + (row_idx * columns)])),
+                        columns - indent);
     row_str = trim(row_str);
-    // End of block
-    if (blank_row_count > 1 || row_str == "(end)") break;
-
-    // End of menu
-    if (row_str.find("(end)") != std::string::npos) break;
-
-    if (row_str.size() == 0)
-      blank_row_count++;
-    else {
-      if (row_idx > 0) {
-        output += "\n";
-      }
-      output += row_str;
-      blank_row_count = 0;
+    if (row_idx == 0) {
+      first_row_str = row_str;
+      if (first_row_str == "") return py::bytes("");
     }
-    // End of more
-    if (row_str.find("--More--") != std::string::npos) break;
+    size_t of_pos = row_str.find(" of ");
+    multipage_message =
+        (((of_pos != std::string::npos) && (row_str[of_pos - 2] == '(') &&
+          (row_str[of_pos + 5] == ')')) ||
+         (row_str.find("--More--") != std::string::npos) ||
+         (row_str.find("(end)") != std::string::npos));
+    output += row_str;
+    if (multipage_message) return py::bytes(output);
+    output += "\n";
   }
-  return py::bytes(output);
+  return py::bytes(first_row_str);
 }
 
 }  // namespace nle_language_obsv
